/*	Copyright (c) 2012, Intel Corporation
	All rights reserved.
	Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
		Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
		Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
		Neither the name of the Intel Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.*/

#include <string.h>
#include <jni.h>
#include <cpu-features.h>
#include <arch.h>

extern "C" {
    JNIEXPORT jstring JNICALL Java_com_intel_cpufeatures_MainActivity_getCPUFeatures( JNIEnv* env, jobject thiz );
};


JNIEXPORT jstring JNICALL Java_com_intel_cpufeatures_MainActivity_getCPUFeatures( JNIEnv* env,
                                                  jobject thiz )
{
	char buf[300];

		//init buffer
	buf[0] = '\0';

		//Compiler MACRO
	strlcat(buf, "Compiler Macro: \n", sizeof(buf));
#ifdef __i386__
	strlcat(buf, "__i386__", sizeof(buf));
#endif
#ifdef __arm__
	strlcat(buf, "__arm__", sizeof(buf));
#endif
#ifdef _MIPS_ARCH
	strlcat(buf, "_MIPS_ARCH", sizeof(buf));
#endif


		//Make System
	strlcat(buf, "\n\nMake System: \n", sizeof(buf));
	strlcat(buf, getTarget(), sizeof(buf));


		//CPU Family
	strlcat(buf, "\n\nandroid_getCpuFamily: \n", sizeof(buf));
	AndroidCpuFamily family = android_getCpuFamily();
	if (family == ANDROID_CPU_FAMILY_X86)
		strlcat(buf, "ANDROID_CPU_FAMILY_X86", sizeof(buf));
	if (family == ANDROID_CPU_FAMILY_ARM)
		strlcat(buf, "ANDROID_CPU_FAMILY_ARM", sizeof(buf));
	if (family == ANDROID_CPU_FAMILY_MIPS)
		strlcat(buf, "ANDROID_CPU_FAMILY_MIPS", sizeof(buf));
	if (family == ANDROID_CPU_FAMILY_UNKNOWN)
		strlcat(buf, "ANDROID_CPU_FAMILY_UNKNOWN", sizeof(buf));


		//CPU features
	strlcat(buf, "\n\nandroid_getCpuFeatures(): ", sizeof(buf));
	uint64_t features = android_getCpuFeatures();

	if (family == ANDROID_CPU_FAMILY_X86) {
		if (features & ANDROID_CPU_X86_FEATURE_SSSE3)
			strlcat(buf, "\nANDROID_CPU_X86_FEATURE_SSSE3", sizeof(buf));
		if (features & ANDROID_CPU_X86_FEATURE_MOVBE)
			strlcat(buf, "\nANDROID_CPU_X86_FEATURE_MOVBE", sizeof(buf));
		if (features & ANDROID_CPU_X86_FEATURE_POPCNT)
			strlcat(buf, "\nANDROID_CPU_X86_FEATURE_POPCNT", sizeof(buf));

	} else if (family == ANDROID_CPU_FAMILY_ARM) {
		if (features & ANDROID_CPU_ARM_FEATURE_ARMv7)
			strlcat(buf, "\nANDROID_CPU_ARM_FEATURE_ARMv7", sizeof(buf));
		if (features & ANDROID_CPU_ARM_FEATURE_VFPv3)
			strlcat(buf, "\nANDROID_CPU_ARM_FEATURE_VFPv3", sizeof(buf));
		if (features & ANDROID_CPU_ARM_FEATURE_NEON)
			strlcat(buf, "\nANDROID_CPU_ARM_FEATURE_NEON", sizeof(buf));
		if (features & ANDROID_CPU_ARM_FEATURE_LDREX_STREX)
			strlcat(buf, "\nANDROID_CPU_ARM_FEATURE_LDREX_STREX", sizeof(buf));
	}


	return env->NewStringUTF(buf);
}
